@@ -1,3 +1,6 @@
+#[cfg(test)]
+mod dtls_transport_test;
+
 pub mod dtls_certificate;
 pub mod dtls_fingerprint;
 pub mod dtls_parameters;
@@ -7,32 +10,32 @@ pub mod dtls_transport_state;
 use crate::api::setting_engine::SettingEngine;
 use crate::default_srtp_protection_profiles;
 use crate::error::Error;
+use crate::media::dtls_transport::dtls_parameters::DTLSParameters;
 use crate::media::dtls_transport::dtls_transport_state::DTLSTransportState;
 use crate::media::ice_transport::ice_transport_state::ICETransportState;
 use crate::media::ice_transport::ICETransport;
 use crate::peer::ice::ice_role::ICERole;
+use crate::util::flatten_errs;
 use crate::util::mux::endpoint::Endpoint;
 use crate::util::mux::mux_func::{match_dtls, match_srtcp, match_srtp, MatchFunc};
 
+use crate::media::dtls_transport::dtls_certificate::Certificate;
+use anyhow::Result;
 use bytes::Bytes;
 use dtls::config::ClientAuthType;
 use dtls::conn::DTLSConn;
-use dtls::crypto::Certificate;
 use dtls_role::*;
+use sha2::{Digest, Sha256};
 use srtp::protection_profile::ProtectionProfile;
 use srtp::session::Session;
 use srtp::stream::Stream;
 use std::future::Future;
 use std::pin::Pin;
+use std::sync::atomic::{AtomicBool, AtomicU8, Ordering};
 use std::sync::Arc;
 use tokio::sync::{mpsc, Mutex};
 use util::Conn;
 
-use crate::media::dtls_transport::dtls_parameters::DTLSParameters;
-use crate::util::flatten_errs;
-use anyhow::Result;
-use std::sync::atomic::{AtomicU8, Ordering};
-
 pub type OnDTLSTransportStateChangeHdlrFn = Box<
     dyn (FnMut(DTLSTransportState) -> Pin<Box<dyn Future<Output = ()> + Send + 'static>>)
         + Send
@@ -62,6 +65,8 @@ pub struct DTLSTransport {
     pub(crate) srtcp_endpoint: Mutex<Option<Arc<Endpoint>>>,
 
     pub(crate) simulcast_streams: Mutex<Vec<Arc<Stream>>>,
+
+    pub(crate) srtp_ready_signal: Arc<AtomicBool>,
     pub(crate) srtp_ready_tx: Mutex<Option<mpsc::Sender<()>>>,
     pub(crate) srtp_ready_rx: Mutex<Option<mpsc::Receiver<()>>>,
 
@@ -79,6 +84,7 @@ impl DTLSTransport {
             ice_transport,
             certificates,
             setting_engine,
+            srtp_ready_signal: Arc::new(AtomicBool::new(false)),
             srtp_ready_tx: Mutex::new(Some(srtp_ready_tx)),
             srtp_ready_rx: Mutex::new(Some(srtp_ready_rx)),
             state: AtomicU8::new(DTLSTransportState::New as u8),
@@ -121,7 +127,10 @@ impl DTLSTransport {
 
     /// write_rtcp sends a user provided RTCP packet to the connected peer. If no peer is connected the
     /// packet is discarded.
-    pub async fn write_rtcp(&self, pkt: &(dyn rtcp::packet::Packet)) -> Result<usize> {
+    pub async fn write_rtcp(
+        &self,
+        pkt: &(dyn rtcp::packet::Packet + Send + Sync),
+    ) -> Result<usize> {
         let srtcp_session = self.srtcp_session.lock().await;
         if let Some(srtcp_session) = &*srtcp_session {
             Ok(srtcp_session.write_rtcp(pkt).await?)
@@ -132,11 +141,10 @@ impl DTLSTransport {
 
     /// get_local_parameters returns the DTLS parameters of the local DTLSTransport upon construction.
     pub fn get_local_parameters(&self) -> Result<DTLSParameters> {
-        let fingerprints = vec![];
+        let mut fingerprints = vec![];
 
-        for _c in &self.certificates {
-            /*TODO: prints := c.GetFingerprints()?;
-            fingerprints.push(prints);*/
+        for c in &self.certificates {
+            fingerprints.push(c.get_fingerprint()?);
         }
 
         Ok(DTLSParameters {
@@ -162,10 +170,6 @@ impl DTLSTransport {
             profile,
             ..Default::default()
         };
-        let mut srtcp_config = srtp::config::Config {
-            profile,
-            ..Default::default()
-        };
 
         if self.setting_engine.replay_protection.srtp != 0 {
             srtp_config.remote_rtp_options = Some(srtp::option::srtp_replay_protection(
@@ -175,14 +179,6 @@ impl DTLSTransport {
             srtp_config.remote_rtp_options = Some(srtp::option::srtp_no_replay_protection());
         }
 
-        if self.setting_engine.replay_protection.srtcp != 0 {
-            srtcp_config.remote_rtcp_options = Some(srtp::option::srtcp_replay_protection(
-                self.setting_engine.replay_protection.srtcp,
-            ));
-        } else if self.setting_engine.disable_srtcp_replay_protection {
-            srtcp_config.remote_rtcp_options = Some(srtp::option::srtcp_no_replay_protection());
-        }
-
         if let Some(conn) = self.conn().await {
             let conn_state = conn.connection_state().await;
             srtp_config
@@ -211,6 +207,27 @@ impl DTLSTransport {
             };
         }
 
+        let mut srtcp_config = srtp::config::Config {
+            profile,
+            ..Default::default()
+        };
+        if self.setting_engine.replay_protection.srtcp != 0 {
+            srtcp_config.remote_rtcp_options = Some(srtp::option::srtcp_replay_protection(
+                self.setting_engine.replay_protection.srtcp,
+            ));
+        } else if self.setting_engine.disable_srtcp_replay_protection {
+            srtcp_config.remote_rtcp_options = Some(srtp::option::srtcp_no_replay_protection());
+        }
+
+        if let Some(conn) = self.conn().await {
+            let conn_state = conn.connection_state().await;
+            srtcp_config
+                .extract_session_keys_from_dtls(conn_state, self.role().await == DTLSRole::Client)
+                .await?;
+        } else {
+            return Err(Error::ErrDtlsTransportNotStarted.into());
+        }
+
         {
             let mut srtcp_session = self.srtcp_session.lock().await;
             *srtcp_session = {
@@ -233,6 +250,9 @@ impl DTLSTransport {
         {
             let mut srtp_ready_tx = self.srtp_ready_tx.lock().await;
             srtp_ready_tx.take();
+            if srtp_ready_tx.is_none() {
+                self.srtp_ready_signal.store(true, Ordering::SeqCst);
+            }
         }
 
         Ok(())
@@ -297,13 +317,17 @@ impl DTLSTransport {
             *rp = remote_parameters;
         }
 
-        let cert = self.certificates[0].clone();
+        let certificate = if let Some(cert) = self.certificates.first() {
+            cert.certificate.clone()
+        } else {
+            return Err(Error::ErrNonCertificate.into());
+        };
         self.state_change(DTLSTransportState::Connecting).await;
 
         Ok((
             self.role().await,
             dtls::config::Config {
-                certificates: vec![cert],
+                certificates: vec![certificate],
                 srtp_protection_profiles: if !self
                     .setting_engine
                     .srtp_protection_profiles
@@ -391,30 +415,21 @@ impl DTLSTransport {
             self.state_change(DTLSTransportState::Failed).await;
             return Err(Error::ErrNoRemoteCertificate.into());
         }
+
         {
             let mut remote_certificate = self.remote_certificate.lock().await;
             *remote_certificate = Bytes::from(remote_certs[0].clone());
         }
 
-        /*TODO: let parsedRemoteCert = x509.ParseCertificate(t.remote_certificate)
-        if err != nil {
-            if closeErr := dtlsConn.Close(); closeErr != nil {
-                t.log.Error(err.Error())
+        if let Err(err) = self.validate_fingerprint(&remote_certs[0]).await {
+            if dtls_conn.close().await.is_err() {
+                log::error!("{}", err);
             }
 
-            t.onStateChange(DTLSTransportStateFailed)
-            return err
+            self.state_change(DTLSTransportState::Failed).await;
+            return Err(err);
         }
 
-        if err = t.validate_finger_print(parsedRemoteCert); err != nil {
-            if closeErr := dtlsConn.Close(); closeErr != nil {
-                t.log.Error(err.Error())
-            }
-
-            t.onStateChange(DTLSTransportStateFailed)
-            return err
-        }*/
-
         {
             let mut conn = self.conn.lock().await;
             *conn = Some(Arc::new(dtls_conn));
@@ -481,25 +496,25 @@ impl DTLSTransport {
         flatten_errs(close_errs)
     }
 
-    pub(crate) fn validate_fingerprint(&self, _remote_cert: &[u8]) -> Result<()> {
-        /*TODO: for  fp in self.remote_parameters.fingerprints {
-            hashAlgo, err := fingerprint.HashFromString(fp.algorithm);
-            if err != nil {
-                return err
+    pub(crate) async fn validate_fingerprint(&self, remote_cert: &[u8]) -> Result<()> {
+        let remote_parameters = self.remote_parameters.lock().await;
+        for fp in &remote_parameters.fingerprints {
+            if fp.algorithm != "sha-256" {
+                return Err(Error::ErrUnsupportedFingerprintAlgorithm.into());
             }
 
-            remoteValue, err := fingerprint.Fingerprint(remoteCert, hashAlgo)
-            if err != nil {
-                return err
-            }
+            let mut h = Sha256::new();
+            h.update(remote_cert);
+            let hashed = h.finalize();
+            let values: Vec<String> = hashed.iter().map(|x| format! {"{:02x}", x}).collect();
+            let remote_value = values.join(":").to_lowercase();
 
-            if strings.EqualFold(remoteValue, fp.Value) {
-                return nil
+            if remote_value == fp.value.to_lowercase() {
+                return Ok(());
             }
         }
 
-        return errNoMatchingCertificateFingerprint*/
-        Ok(())
+        Err(Error::ErrNoMatchingCertificateFingerprint.into())
     }
 
     pub(crate) fn ensure_ice_conn(&self) -> Result<()> {
@@ -515,20 +530,3 @@ impl DTLSTransport {
         simulcast_streams.push(stream)
     }
 }
-
-#[cfg(test)]
-mod test {
-    use super::*;
-
-    #[tokio::test]
-    async fn test_invalid_fingerprint_causes_failed() -> Result<()> {
-        //TODO:
-        Ok(())
-    }
-
-    #[tokio::test]
-    async fn test_peer_connection_dtls_role_setting_engine() -> Result<()> {
-        //TODO:
-        Ok(())
-    }
-}
