@@ -4,7 +4,6 @@ mod rtp_receiver_test;
 use crate::api::media_engine::MediaEngine;
 use crate::error::Error;
 use crate::media::dtls_transport::DTLSTransport;
-use crate::media::interceptor::stream_info::{RTPHeaderExtension, StreamInfo};
 use crate::media::interceptor::*;
 use crate::media::rtp::rtp_codec::{
     codec_parameters_fuzzy_search, CodecMatch, RTPCodecCapability, RTPCodecParameters,
@@ -19,31 +18,34 @@ use crate::RECEIVE_MTU;
 
 use crate::peer::sdp::TrackDetails;
 use anyhow::Result;
+use interceptor::stream_info::{RTPHeaderExtension, StreamInfo};
+use interceptor::{Attributes, Interceptor, RTCPReader, RTPReader};
 use std::sync::Arc;
 use tokio::sync::{mpsc, Mutex};
 
 pub(crate) struct RTPReceiverInternal {
     pub(crate) kind: RTPCodecType,
-    tracks: Vec<TrackStreams>,
+    tracks: Mutex<Vec<TrackStreams>>,
+    closed_rx: Mutex<mpsc::Receiver<()>>,
+    received_rx: Mutex<mpsc::Receiver<()>>,
+
+    transceiver_codecs: Mutex<Option<Arc<Mutex<Vec<RTPCodecParameters>>>>>,
 
     transport: Arc<DTLSTransport>,
     media_engine: Arc<MediaEngine>,
-    interceptor: Option<Arc<dyn Interceptor + Send + Sync>>,
-
-    closed_tx: Option<mpsc::Sender<()>>,
-    closed_rx: mpsc::Receiver<()>,
-    received_tx: Option<mpsc::Sender<()>>,
-    received_rx: mpsc::Receiver<()>,
-
-    transceiver_codecs: Option<Arc<Mutex<Vec<RTPCodecParameters>>>>,
+    interceptor: Arc<dyn Interceptor + Send + Sync>,
 }
 
 impl RTPReceiverInternal {
     /// read reads incoming RTCP for this RTPReceiver
-    async fn read(&mut self, b: &mut [u8]) -> Result<(usize, Attributes)> {
+    async fn read(&self, b: &mut [u8]) -> Result<(usize, Attributes)> {
+        let (mut received_rx, mut closed_rx) =
+            (self.received_rx.lock().await, self.closed_rx.lock().await);
+
         tokio::select! {
-            _ = self.received_rx.recv() =>{
-                if let Some(t) = self.tracks.first(){
+            _ = received_rx.recv() =>{
+                let tracks = self.tracks.lock().await;
+                if let Some(t) = tracks.first(){
                     if let Some(rtcp_interceptor) = &t.rtcp_interceptor{
                         let a = Attributes::new();
                         rtcp_interceptor.read(b, &a).await
@@ -54,17 +56,21 @@ impl RTPReceiverInternal {
                     Err(Error::ErrExistingTrack.into())
                 }
             }
-            _ = self.closed_rx.recv() => {
+            _ = closed_rx.recv() => {
                 Err(Error::ErrClosedPipe.into())
             }
         }
     }
 
     /// read_simulcast reads incoming RTCP for this RTPReceiver for given rid
-    async fn read_simulcast(&mut self, b: &mut [u8], rid: &str) -> Result<(usize, Attributes)> {
+    async fn read_simulcast(&self, b: &mut [u8], rid: &str) -> Result<(usize, Attributes)> {
+        let (mut received_rx, mut closed_rx) =
+            (self.received_rx.lock().await, self.closed_rx.lock().await);
+
         tokio::select! {
-            _ = self.received_rx.recv() =>{
-                for t in &self.tracks{
+            _ = received_rx.recv() =>{
+                let tracks = self.tracks.lock().await;
+                for t in &*tracks{
                     if t.track.rid() == rid {
                        if let Some(rtcp_interceptor) = &t.rtcp_interceptor{
                             let a = Attributes::new();
@@ -76,7 +82,7 @@ impl RTPReceiverInternal {
                 }
                 Err(Error::ErrRTPReceiverForRIDTrackStreamNotFound.into())
             }
-            _ = self.closed_rx.recv() => {
+            _ = closed_rx.recv() => {
                 Err(Error::ErrClosedPipe.into())
             }
         }
@@ -84,7 +90,7 @@ impl RTPReceiverInternal {
 
     /// read_rtcp is a convenience method that wraps Read and unmarshal for you.
     /// It also runs any configured interceptors.
-    async fn read_rtcp(&mut self) -> Result<(Box<dyn rtcp::packet::Packet>, Attributes)> {
+    async fn read_rtcp(&self) -> Result<(Box<dyn rtcp::packet::Packet>, Attributes)> {
         let mut b = vec![0u8; RECEIVE_MTU];
         let (n, attributes) = self.read(&mut b).await?;
 
@@ -96,7 +102,7 @@ impl RTPReceiverInternal {
 
     /// read_simulcast_rtcp is a convenience method that wraps ReadSimulcast and unmarshal for you
     async fn read_simulcast_rtcp(
-        &mut self,
+        &self,
         rid: &str,
     ) -> Result<(Box<dyn rtcp::packet::Packet>, Attributes)> {
         let mut b = vec![0u8; RECEIVE_MTU];
@@ -108,23 +114,31 @@ impl RTPReceiverInternal {
         Ok((pkts, attributes))
     }
 
-    pub(crate) async fn read_rtp(
-        &mut self,
-        b: &mut [u8],
-        tid: &str,
-    ) -> Result<(usize, Attributes)> {
-        let _ = self.received_rx.recv().await;
-
-        for t in &self.tracks {
-            if t.track.id().await == tid {
-                if let Some(ri) = &t.rtp_interceptor {
-                    let a = Attributes::new();
-                    return ri.read(b, &a).await;
-                }
-            }
+    pub(crate) async fn read_rtp(&self, b: &mut [u8], tid: &str) -> Result<(usize, Attributes)> {
+        {
+            let mut received_rx = self.received_rx.lock().await;
+            let _ = received_rx.recv().await;
         }
 
-        Err(Error::ErrRTPReceiverWithSSRCTrackStreamNotFound.into())
+        log::debug!("read_rtp enter tracks");
+        let mut rtp_interceptor = None;
+        {
+            let tracks = self.tracks.lock().await;
+            for t in &*tracks {
+                if t.track.id().await == tid {
+                    rtp_interceptor = t.rtp_interceptor.clone();
+                    break;
+                }
+            }
+        };
+        log::debug!("read_rtp exit tracks");
+
+        if let Some(ri) = rtp_interceptor {
+            let a = Attributes::new();
+            ri.read(b, &a).await
+        } else {
+            Err(Error::ErrRTPReceiverWithSSRCTrackStreamNotFound.into())
+        }
     }
 
     async fn get_parameters(&self) -> RTPParameters {
@@ -133,7 +147,8 @@ impl RTPReceiverInternal {
             .get_rtp_parameters_by_kind(self.kind, &[RTPTransceiverDirection::Recvonly])
             .await;
 
-        if let Some(codecs) = &self.transceiver_codecs {
+        let transceiver_codecs = self.transceiver_codecs.lock().await;
+        if let Some(codecs) = &*transceiver_codecs {
             let c = codecs.lock().await;
             parameters.codecs =
                 RTPReceiverInternal::get_codecs(&*c, self.kind, &self.media_engine).await;
@@ -167,7 +182,10 @@ impl RTPReceiverInternal {
 pub struct RTPReceiver {
     kind: RTPCodecType,
     transport: Arc<DTLSTransport>,
-    pub(crate) internal: Arc<Mutex<RTPReceiverInternal>>,
+    closed_tx: Mutex<Option<mpsc::Sender<()>>>,
+    received_tx: Mutex<Option<mpsc::Sender<()>>>,
+
+    pub(crate) internal: Arc<RTPReceiverInternal>,
 }
 
 impl RTPReceiver {
@@ -175,7 +193,7 @@ impl RTPReceiver {
         kind: RTPCodecType,
         transport: Arc<DTLSTransport>,
         media_engine: Arc<MediaEngine>,
-        interceptor: Option<Arc<dyn Interceptor + Send + Sync>>,
+        interceptor: Arc<dyn Interceptor + Send + Sync>,
     ) -> Self {
         let (closed_tx, closed_rx) = mpsc::channel(1);
         let (received_tx, received_rx) = mpsc::channel(1);
@@ -183,21 +201,22 @@ impl RTPReceiver {
         RTPReceiver {
             kind,
             transport: Arc::clone(&transport),
-            internal: Arc::new(Mutex::new(RTPReceiverInternal {
+            closed_tx: Mutex::new(Some(closed_tx)),
+            received_tx: Mutex::new(Some(received_tx)),
+
+            internal: Arc::new(RTPReceiverInternal {
                 kind,
 
-                tracks: vec![],
+                tracks: Mutex::new(vec![]),
                 transport,
                 media_engine,
                 interceptor,
 
-                closed_tx: Some(closed_tx),
-                closed_rx,
-                received_tx: Some(received_tx),
-                received_rx,
+                closed_rx: Mutex::new(closed_rx),
+                received_rx: Mutex::new(received_rx),
 
-                transceiver_codecs: None,
-            })),
+                transceiver_codecs: Mutex::new(None),
+            }),
         }
     }
 
@@ -209,8 +228,8 @@ impl RTPReceiver {
         &self,
         codecs: Option<Arc<Mutex<Vec<RTPCodecParameters>>>>,
     ) {
-        let mut internal = self.internal.lock().await;
-        internal.transceiver_codecs = codecs;
+        let mut transceiver_codecs = self.internal.transceiver_codecs.lock().await;
+        *transceiver_codecs = codecs;
     }
 
     /// transport returns the currently-configured *DTLSTransport or nil
@@ -222,8 +241,7 @@ impl RTPReceiver {
     /// get_parameters describes the current configuration for the encoding and
     /// transmission of media on the receiver's track.
     pub async fn get_parameters(&self) -> RTPParameters {
-        let internal = self.internal.lock().await;
-        internal.get_parameters().await
+        self.internal.get_parameters().await
     }
 
     /// SetRTPParameters applies provided RTPParameters the RTPReceiver's tracks.
@@ -239,9 +257,9 @@ impl RTPReceiver {
             });
         }
 
-        let mut internal = self.internal.lock().await;
+        let mut tracks = self.internal.tracks.lock().await;
         for (idx, codec) in params.codecs.iter().enumerate() {
-            let t = &mut internal.tracks[idx];
+            let t = &mut tracks[idx];
             t.stream_info.rtp_header_extensions = header_extensions.clone();
 
             let current_track = &t.track;
@@ -252,31 +270,38 @@ impl RTPReceiver {
 
     /// track returns the RtpTransceiver TrackRemote
     pub async fn track(&self) -> Option<Arc<TrackRemote>> {
-        let internal = self.internal.lock().await;
-        internal.tracks.first().map(|t| Arc::clone(&t.track))
+        let tracks = self.internal.tracks.lock().await;
+        tracks.first().map(|t| Arc::clone(&t.track))
     }
 
     /// tracks returns the RtpTransceiver tracks
     /// A RTPReceiver to support Simulcast may now have multiple tracks
     pub async fn tracks(&self) -> Vec<Arc<TrackRemote>> {
-        let internal = self.internal.lock().await;
-        internal
-            .tracks
-            .iter()
-            .map(|t| Arc::clone(&t.track))
-            .collect()
+        let tracks = self.internal.tracks.lock().await;
+        tracks.iter().map(|t| Arc::clone(&t.track)).collect()
     }
 
     /// receive initialize the track and starts all the transports
     pub async fn receive(&self, parameters: &RTPReceiveParameters) -> Result<()> {
         let receiver = Arc::clone(&self.internal);
 
-        let mut internal = self.internal.lock().await;
-        if internal.received_tx.is_none() {
-            return Err(Error::ErrRTPReceiverReceiveAlreadyCalled.into());
-        }
-        let (_d, global_params) = (internal.received_tx.take(), internal.get_parameters().await);
+        let _d = {
+            let mut received_tx = self.received_tx.lock().await;
+            if received_tx.is_none() {
+                return Err(Error::ErrRTPReceiverReceiveAlreadyCalled.into());
+            }
+            received_tx.take()
+        };
 
+        let (global_params, interceptor, media_engine) = {
+            (
+                self.internal.get_parameters().await,
+                Arc::clone(&self.internal.interceptor),
+                Arc::clone(&self.internal.media_engine),
+            )
+        };
+
+        let mut tracks = vec![];
         if parameters.encodings.len() == 1 && parameters.encodings[0].ssrc != 0 {
             if let Some(encoding) = parameters.encodings.first() {
                 let codec = if let Some(codec) = global_params.codecs.first() {
@@ -285,7 +310,7 @@ impl RTPReceiver {
                     RTPCodecCapability::default()
                 };
 
-                let stream_info = StreamInfo::new(
+                let stream_info = create_stream_info(
                     "".to_owned(),
                     encoding.ssrc,
                     0,
@@ -293,8 +318,13 @@ impl RTPReceiver {
                     &global_params.header_extensions,
                 );
                 let (rtp_read_stream, rtp_interceptor, rtcp_read_stream, rtcp_interceptor) =
-                    RTPReceiver::streams_for_ssrc(&self.transport, encoding.ssrc, &stream_info)
-                        .await?;
+                    RTPReceiver::streams_for_ssrc(
+                        &self.transport,
+                        encoding.ssrc,
+                        &stream_info,
+                        &interceptor,
+                    )
+                    .await?;
 
                 let t = TrackStreams {
                     track: Arc::new(TrackRemote::new(
@@ -302,8 +332,8 @@ impl RTPReceiver {
                         encoding.ssrc,
                         "".to_owned(),
                         receiver,
-                        Arc::clone(&internal.media_engine),
-                        internal.interceptor.clone(),
+                        Arc::clone(&media_engine),
+                        Arc::clone(&interceptor),
                     )),
                     stream_info,
                     rtp_read_stream,
@@ -312,7 +342,7 @@ impl RTPReceiver {
                     rtcp_interceptor,
                 };
 
-                internal.tracks.push(t);
+                tracks.push(t);
             }
         } else {
             for encoding in &parameters.encodings {
@@ -322,36 +352,42 @@ impl RTPReceiver {
                         0,
                         encoding.rid.clone(),
                         Arc::clone(&receiver),
-                        Arc::clone(&internal.media_engine),
-                        internal.interceptor.clone(),
+                        Arc::clone(&media_engine),
+                        Arc::clone(&interceptor),
                     )),
-                    ..Default::default()
+                    stream_info: Default::default(),
+                    rtp_read_stream: None,
+                    rtp_interceptor: None,
+                    rtcp_read_stream: None,
+                    rtcp_interceptor: None,
                 };
 
-                internal.tracks.push(t);
+                tracks.push(t);
             }
         }
 
+        {
+            let mut internal_tracks = self.internal.tracks.lock().await;
+            internal_tracks.extend(tracks);
+        };
+
         Ok(())
     }
 
     /// read reads incoming RTCP for this RTPReceiver
     pub async fn read(&self, b: &mut [u8]) -> Result<(usize, Attributes)> {
-        let mut internal = self.internal.lock().await;
-        internal.read(b).await
+        self.internal.read(b).await
     }
 
     /// read_simulcast reads incoming RTCP for this RTPReceiver for given rid
     pub async fn read_simulcast(&self, b: &mut [u8], rid: &str) -> Result<(usize, Attributes)> {
-        let mut internal = self.internal.lock().await;
-        internal.read_simulcast(b, rid).await
+        self.internal.read_simulcast(b, rid).await
     }
 
     /// read_rtcp is a convenience method that wraps Read and unmarshal for you.
     /// It also runs any configured interceptors.
     pub async fn read_rtcp(&self) -> Result<(Box<dyn rtcp::packet::Packet>, Attributes)> {
-        let mut internal = self.internal.lock().await;
-        internal.read_rtcp().await
+        self.internal.read_rtcp().await
     }
 
     /// read_simulcast_rtcp is a convenience method that wraps ReadSimulcast and unmarshal for you
@@ -359,13 +395,12 @@ impl RTPReceiver {
         &self,
         rid: &str,
     ) -> Result<(Box<dyn rtcp::packet::Packet>, Attributes)> {
-        let mut internal = self.internal.lock().await;
-        internal.read_simulcast_rtcp(rid).await
+        self.internal.read_simulcast_rtcp(rid).await
     }
 
     pub(crate) async fn have_received(&self) -> bool {
-        let internal = self.internal.lock().await;
-        internal.received_tx.is_none()
+        let received_tx = self.received_tx.lock().await;
+        received_tx.is_none()
     }
 
     pub(crate) async fn start(&self, incoming: &TrackDetails) -> bool {
@@ -397,58 +432,27 @@ impl RTPReceiver {
 
         // We can't block and wait for a single SSRC
         incoming.ssrc != 0
-
-        /*
-        let media_engine = Arc::clone(&self.media_engine);
-        tokio::spawn(async move {
-            if let Some(track) = receiver.track().await {
-                if let Err(err) = track.determine_payload_type().await {
-                    log::warn!(
-                        "Could not determine PayloadType for SSRC {} with err {}",
-                        track.ssrc(),
-                        err
-                    );
-                    return;
-                }
-
-                let params = match media_engine
-                    .get_rtp_parameters_by_payload_type(track.payload_type())
-                    .await
-                {
-                    Ok(params) => params,
-                    Err(err) => {
-                        log::warn!(
-                            "no codec could be found for payloadType {} with err {}",
-                            track.payload_type(),
-                            err,
-                        );
-                        return;
-                    }
-                };
-
-                track.set_kind(receiver.kind());
-                track.set_codec(params.codecs[0].clone()).await;
-                track.set_params(params).await;
-
-                //TODO:self.do_track(receiver.track().await, Some(Arc::clone(&receiver))).await;
-            }
-        });*/
     }
 
     /// Stop irreversibly stops the RTPReceiver
     pub async fn stop(&self) -> Result<()> {
-        let mut internal = self.internal.lock().await;
-
         let _d = {
-            if internal.closed_tx.is_none() {
+            let mut closed_tx = self.closed_tx.lock().await;
+            if closed_tx.is_none() {
                 return Ok(());
             }
-            internal.closed_tx.take()
+            closed_tx.take()
+        };
+
+        let received_tx_is_none = {
+            let received_tx = self.received_tx.lock().await;
+            received_tx.is_none()
         };
 
         let mut errs = vec![];
-        if internal.received_tx.is_none() {
-            for t in &internal.tracks {
+        if received_tx_is_none {
+            let tracks = self.internal.tracks.lock().await;
+            for t in &*tracks {
                 if let Some(rtcp_read_stream) = &t.rtcp_read_stream {
                     if let Err(err) = rtcp_read_stream.close().await {
                         errs.push(err);
@@ -461,9 +465,10 @@ impl RTPReceiver {
                     }
                 }
 
-                if let Some(interceptor) = &internal.interceptor {
-                    interceptor.unbind_remote_stream(&t.stream_info).await;
-                }
+                self.internal
+                    .interceptor
+                    .unbind_remote_stream(&t.stream_info)
+                    .await;
             }
         }
 
@@ -472,8 +477,7 @@ impl RTPReceiver {
 
     /// read_rtp should only be called by a track, this only exists so we can keep state in one place
     pub(crate) async fn read_rtp(&self, b: &mut [u8], tid: &str) -> Result<(usize, Attributes)> {
-        let mut internal = self.internal.lock().await;
-        internal.read_rtp(b, tid).await
+        self.internal.read_rtp(b, tid).await
     }
 
     /// receive_for_rid is the sibling of Receive expect for RIDs instead of SSRCs
@@ -484,40 +488,53 @@ impl RTPReceiver {
         params: &RTPParameters,
         ssrc: SSRC,
     ) -> Result<Arc<TrackRemote>> {
-        let mut internal = self.internal.lock().await;
-        for t in &mut internal.tracks {
-            if t.track.rid() == rid && !params.codecs.is_empty() {
-                t.track.set_kind(self.kind);
-                t.track.set_codec(params.codecs[0].clone()).await;
-                t.track.set_params(params.clone()).await;
-                t.track.set_ssrc(ssrc);
-                t.stream_info = StreamInfo::new(
-                    "".to_owned(),
-                    ssrc,
-                    params.codecs[0].payload_type,
-                    params.codecs[0].capability.clone(),
-                    &params.header_extensions,
-                );
+        let interceptor = Arc::clone(&self.internal.interceptor);
+        log::debug!("receive_for_rid enter tracks");
+        {
+            let mut tracks = self.internal.tracks.lock().await;
+            for t in &mut *tracks {
+                if t.track.rid() == rid && !params.codecs.is_empty() {
+                    t.track.set_kind(self.kind);
+                    t.track.set_codec(params.codecs[0].clone()).await;
+                    t.track.set_params(params.clone()).await;
+                    t.track.set_ssrc(ssrc);
+                    t.stream_info = create_stream_info(
+                        "".to_owned(),
+                        ssrc,
+                        params.codecs[0].payload_type,
+                        params.codecs[0].capability.clone(),
+                        &params.header_extensions,
+                    );
 
-                let (rtp_read_stream, rtp_interceptor, rtcp_read_stream, rtcp_interceptor) =
-                    RTPReceiver::streams_for_ssrc(&self.transport, ssrc, &t.stream_info).await?;
+                    let (rtp_read_stream, rtp_interceptor, rtcp_read_stream, rtcp_interceptor) =
+                        RTPReceiver::streams_for_ssrc(
+                            &self.transport,
+                            ssrc,
+                            &t.stream_info,
+                            &interceptor,
+                        )
+                        .await?;
 
-                t.rtp_read_stream = rtp_read_stream;
-                t.rtp_interceptor = rtp_interceptor;
-                t.rtcp_read_stream = rtcp_read_stream;
-                t.rtcp_interceptor = rtcp_interceptor;
+                    t.rtp_read_stream = rtp_read_stream;
+                    t.rtp_interceptor = rtp_interceptor;
+                    t.rtcp_read_stream = rtcp_read_stream;
+                    t.rtcp_interceptor = rtcp_interceptor;
 
-                return Ok(Arc::clone(&t.track));
+                    log::debug!("receive_for_rid exit tracks 1");
+                    return Ok(Arc::clone(&t.track));
+                }
             }
         }
 
+        log::debug!("receive_for_rid exit tracks 2");
         Err(Error::ErrRTPReceiverForSSRCTrackStreamNotFound.into())
     }
 
     async fn streams_for_ssrc(
         transport: &Arc<DTLSTransport>,
         ssrc: SSRC,
-        _stream_info: &StreamInfo,
+        stream_info: &StreamInfo,
+        interceptor: &Arc<dyn Interceptor + Send + Sync>,
     ) -> Result<(
         Option<Arc<srtp::stream::Stream>>,
         Option<Arc<dyn RTPReader + Send + Sync>>,
@@ -528,31 +545,25 @@ impl RTPReceiver {
             .get_srtp_session()
             .await
             .ok_or(Error::ErrDtlsTransportNotStarted)?;
-        let rtp_read_stream = srtp_session.listen(ssrc).await?;
-
-        /*TODO: rtp_interceptor := r.api.interceptor.bind_remote_stream(&streamInfo, interceptor.RTPReaderFunc(func(in []byte, a interceptor.Attributes) (n int, attributes interceptor.Attributes, err error) {
-            n, err = rtp_read_stream.Read(in)
-            return n, a, err
-        }))*/
-        let rtp_interceptor = None;
+        let rtp_read_stream = Arc::new(srtp_session.listen(ssrc).await?);
+        let rtp_stream_reader = Arc::clone(&rtp_read_stream) as Arc<dyn RTPReader + Send + Sync>;
+        let rtp_interceptor = interceptor
+            .bind_remote_stream(stream_info, rtp_stream_reader)
+            .await;
 
         let srtcp_session = transport
             .get_srtcp_session()
             .await
             .ok_or(Error::ErrDtlsTransportNotStarted)?;
-        let rtcp_read_stream = srtcp_session.listen(ssrc).await?;
-
-        /*TODO: rtcp_interceptor := r.api.interceptor.bind_rtcpreader(interceptor.RTPReaderFunc(func(in []byte, a interceptor.Attributes) (n int, attributes interceptor.Attributes, err error) {
-            n, err = rtcp_read_stream.Read(in)
-            return n, a, err
-        }))*/
-        let rtcp_interceptor = None;
+        let rtcp_read_stream = Arc::new(srtcp_session.listen(ssrc).await?);
+        let rtcp_stream_reader = Arc::clone(&rtcp_read_stream) as Arc<dyn RTCPReader + Send + Sync>;
+        let rtcp_interceptor = interceptor.bind_rtcp_reader(rtcp_stream_reader).await;
 
         Ok((
-            Some(Arc::new(rtp_read_stream)),
-            rtp_interceptor,
-            Some(Arc::new(rtcp_read_stream)),
-            rtcp_interceptor,
+            Some(rtp_read_stream),
+            Some(rtp_interceptor),
+            Some(rtcp_read_stream),
+            Some(rtcp_interceptor),
         ))
     }
 }
