@@ -174,8 +174,26 @@ async fn test_track_local_static_closed() -> Result<()> {
     Ok(())
 }
 
+//use log::LevelFilter;
+//use std::io::Write;
+
 #[tokio::test]
 async fn test_track_local_static_payload_type() -> Result<()> {
+    /*env_logger::Builder::new()
+    .format(|buf, record| {
+        writeln!(
+            buf,
+            "{}:{} [{}] {} - {}",
+            record.file().unwrap_or("unknown"),
+            record.line().unwrap_or(0),
+            record.level(),
+            chrono::Local::now().format("%H:%M:%S.%6f"),
+            record.args()
+        )
+    })
+    .filter(None, LevelFilter::Trace)
+    .init();*/
+
     let mut media_engine_one = MediaEngine::default();
     media_engine_one.register_codec(
         RTPCodecParameters {
@@ -219,7 +237,7 @@ async fn test_track_local_static_payload_type() -> Result<()> {
         .new_peer_connection(Configuration::default())
         .await?;
 
-    let track: Arc<dyn TrackLocal + Send + Sync> = Arc::new(TrackLocalStaticSample::new(
+    let track = Arc::new(TrackLocalStaticSample::new(
         RTPCodecCapability {
             mime_type: "video/vp8".to_owned(),
             ..Default::default()
@@ -231,9 +249,11 @@ async fn test_track_local_static_payload_type() -> Result<()> {
         .add_transceiver_from_kind(RTPCodecType::Video, &[])
         .await?;
 
-    answerer.add_track(Arc::clone(&track)).await?;
+    answerer
+        .add_track(Arc::clone(&track) as Arc<dyn TrackLocal + Send + Sync>)
+        .await?;
 
-    let (on_track_fired_tx, _on_track_fired_rx) = mpsc::channel::<()>(1);
+    let (on_track_fired_tx, on_track_fired_rx) = mpsc::channel::<()>(1);
     let on_track_fired_tx = Arc::new(Mutex::new(Some(on_track_fired_tx)));
     offerer
         .on_track(Box::new(
@@ -245,6 +265,7 @@ async fn test_track_local_static_payload_type() -> Result<()> {
                         assert_eq!(t.codec().await.capability.mime_type, "video/VP8");
                     }
                     {
+                        log::debug!("onTrackFiredFunc!!!");
                         let mut done = on_track_fired_tx2.lock().await;
                         done.take();
                     }
@@ -255,8 +276,7 @@ async fn test_track_local_static_payload_type() -> Result<()> {
 
     signal_pair(&mut offerer, &mut answerer).await?;
 
-    //TODO: send_video_until_done(on_track_fired_rx, &[track]).await;
-    //  only works when "if let Err(err) = self.handle_undeclared_ssrc" is done
+    send_video_until_done(on_track_fired_rx, vec![track], Bytes::from_static(&[0x00])).await;
 
     close_pair_now(&offerer, &answerer).await;
 
@@ -323,6 +343,9 @@ async fn test_track_local_static_mutate_input() -> Result<()> {
     Ok(())
 }
 
+//use std::io::Write;
+//use log::LevelFilter;
+
 // Assert that writing to a Track that has Binded (but not connected)
 // does not block
 #[tokio::test]
