@@ -101,23 +101,14 @@ impl ICEGatherer {
             urls: self.validated_servers.clone(),
             port_min: self.setting_engine.ephemeral_udp.port_min,
             port_max: self.setting_engine.ephemeral_udp.port_max,
-            disconnected_timeout: Some(self.setting_engine.timeout.ice_disconnected_timeout),
-            failed_timeout: Some(self.setting_engine.timeout.ice_failed_timeout),
-            keepalive_interval: Some(self.setting_engine.timeout.ice_keepalive_interval),
-            //LoggerFactory:          self.setting_engine.LoggerFactory,
+            disconnected_timeout: self.setting_engine.timeout.ice_disconnected_timeout,
+            failed_timeout: self.setting_engine.timeout.ice_failed_timeout,
+            keepalive_interval: self.setting_engine.timeout.ice_keepalive_interval,
             candidate_types,
-            host_acceptance_min_wait: Some(
-                self.setting_engine.timeout.ice_host_acceptance_min_wait,
-            ),
-            srflx_acceptance_min_wait: Some(
-                self.setting_engine.timeout.ice_srflx_acceptance_min_wait,
-            ),
-            prflx_acceptance_min_wait: Some(
-                self.setting_engine.timeout.ice_prflx_acceptance_min_wait,
-            ),
-            relay_acceptance_min_wait: Some(
-                self.setting_engine.timeout.ice_relay_acceptance_min_wait,
-            ),
+            host_acceptance_min_wait: self.setting_engine.timeout.ice_host_acceptance_min_wait,
+            srflx_acceptance_min_wait: self.setting_engine.timeout.ice_srflx_acceptance_min_wait,
+            prflx_acceptance_min_wait: self.setting_engine.timeout.ice_prflx_acceptance_min_wait,
+            relay_acceptance_min_wait: self.setting_engine.timeout.ice_relay_acceptance_min_wait,
             interface_filter: self.setting_engine.candidates.interface_filter.clone(),
             nat_1to1_ips: self.setting_engine.candidates.nat_1to1_ips.clone(),
             nat_1to1_ip_candidate_type: nat_1to1_cand_type,
@@ -305,8 +296,8 @@ impl ICEGatherer {
         agent.clone()
     }
 
-    /*TODO:
-    func (g *ICEGatherer) collectStats(collector *statsReportCollector) {
+    /*TODO:func (g *ICEGatherer) collectStats(collector *statsReportCollector) {
+
         agent := g.getAgent()
         if agent == nil {
             return
