@@ -12,7 +12,6 @@ use crate::media::dtls_transport::dtls_transport_state::DTLSTransportState;
 use crate::media::dtls_transport::DTLSTransport;
 use crate::media::ice_transport::ice_transport_state::ICETransportState;
 use crate::media::ice_transport::ICETransport;
-use crate::media::interceptor::{Attributes, Interceptor, RTCPWriter};
 use crate::media::rtp::rtp_receiver::RTPReceiver;
 use crate::media::rtp::rtp_transceiver::{
     find_by_mid, handle_unknown_rtp_packet, satisfy_type_and_direction, RTPTransceiver,
@@ -38,6 +37,7 @@ use crate::data::data_channel::data_channel_state::DataChannelState;
 use crate::data::sctp_transport::sctp_transport_capabilities::SCTPTransportCapabilities;
 use crate::data::sctp_transport::sctp_transport_state::SCTPTransportState;
 use crate::error::Error;
+use crate::media::dtls_transport::dtls_certificate::Certificate;
 use crate::media::dtls_transport::dtls_fingerprint::DTLSFingerprint;
 use crate::media::dtls_transport::dtls_parameters::DTLSParameters;
 use crate::media::dtls_transport::dtls_role::{
@@ -63,9 +63,14 @@ use crate::{
     MEDIA_SECTION_APPLICATION, RECEIVE_MTU, SIMULCAST_MAX_PROBE_ROUTINES, SIMULCAST_PROBE_COUNT,
     SSRC_STR,
 };
+
 use anyhow::Result;
+use async_trait::async_trait;
 use ice::candidate::candidate_base::unmarshal_candidate;
 use ice::candidate::Candidate;
+use interceptor::{Attributes, Interceptor, RTCPWriter};
+use peer_connection_internal::*;
+use rcgen::KeyPair;
 use sdp::session_description::{ATTR_KEY_ICELITE, ATTR_KEY_MSID};
 use sdp::util::ConnectionRole;
 use srtp::stream::Stream;
@@ -76,8 +81,6 @@ use std::sync::Arc;
 use std::time::{SystemTime, UNIX_EPOCH};
 use tokio::sync::{mpsc, Mutex};
 
-use peer_connection_internal::*;
-
 pub type OnSignalingStateChangeHdlrFn = Box<
     dyn (FnMut(SignalingState) -> Pin<Box<dyn Future<Output = ()> + Send + 'static>>) + Send + Sync,
 >;
@@ -144,7 +147,6 @@ struct NegotiationNeededParams {
 /// PeerConnection represents a WebRTC connection that establishes a
 /// peer-to-peer communications with another PeerConnection instance in a
 /// browser, or to another endpoint implementing the required protocols.
-#[derive(Default)]
 pub struct PeerConnection {
     stats_id: String,
 
@@ -163,7 +165,7 @@ pub struct PeerConnection {
     /// should be defined (see JSEP 3.4.1).
     greater_mid: isize,
 
-    interceptor_rtcp_writer: Option<Arc<dyn RTCPWriter + Send + Sync>>,
+    interceptor_rtcp_writer: Arc<dyn RTCPWriter + Send + Sync>,
 
     pub(crate) internal: Arc<PeerConnectionInternal>,
 }
@@ -178,6 +180,10 @@ impl PeerConnection {
     pub(crate) async fn new(api: &API, mut configuration: Configuration) -> Result<Self> {
         PeerConnection::init_configuration(&mut configuration)?;
 
+        let internal = Arc::new(PeerConnectionInternal::new(api, &mut configuration).await?);
+        let internal_rtcp_writer = Arc::clone(&internal) as Arc<dyn RTCPWriter + Send + Sync>;
+        let interceptor_rtcp_writer = api.interceptor.bind_rtcp_writer(internal_rtcp_writer).await;
+
         // https://w3c.github.io/webrtc-pc/#constructor (Step #2)
         // Some variables defined explicitly despite their implicit zero values to
         // allow better readability to understand what is happening.
@@ -189,9 +195,11 @@ impl PeerConnection {
             last_offer: "".to_owned(),
             last_answer: "".to_owned(),
             greater_mid: -1,
-            internal: Arc::new(PeerConnectionInternal::new(api, &configuration).await?),
+            interceptor_rtcp_writer,
+            internal,
             configuration,
-            ..Default::default()
+            sdp_origin: Default::default(),
+            idp_login_url: None,
         })
     }
 
@@ -209,25 +217,18 @@ impl PeerConnection {
         }
 
         // https://www.w3.org/TR/webrtc/#constructor (step #3)
-        /*TODO:if !configuration.certificates.is_empty() {
-            now := time.Now()
-            for _, x509Cert := range configuration.Certificates {
-                if !x509Cert.Expires().IsZero() && now.After(x509Cert.Expires()) {
-                    return &rtcerr.InvalidAccessError{Err: ErrCertificateExpired}
+        if !configuration.certificates.is_empty() {
+            let now = SystemTime::now();
+            for cert in &configuration.certificates {
+                if cert.expires().duration_since(now).is_err() {
+                    return Err(Error::ErrCertificateExpired.into());
                 }
-                pc.configuration.Certificates = append(pc.configuration.Certificates, x509Cert)
             }
         } else {
-            sk, err := ecdsa.GenerateKey(elliptic.P256(), rand.Reader)
-            if err != nil {
-                return &rtcerr.UnknownError{Err: err}
-            }
-            certificate, err := GenerateCertificate(sk)
-            if err != nil {
-                return err
-            }
-            pc.configuration.Certificates = []Certificate{*certificate}
-        }  */
+            let kp = KeyPair::generate(&rcgen::PKCS_ECDSA_P256_SHA256)?;
+            let cert = Certificate::from_key_pair(kp)?;
+            configuration.certificates = vec![cert];
+        };
 
         Ok(())
     }
@@ -493,14 +494,7 @@ impl PeerConnection {
         t: Option<Arc<TrackRemote>>,
         r: Option<Arc<RTPReceiver>>,
     ) {
-        log::debug!(
-            "got new track: {}",
-            if let Some(t) = &t {
-                t.id().await
-            } else {
-                "None".to_owned()
-            }
-        );
+        log::debug!("got new track: {:?}", t);
 
         if t.is_some() {
             let mut handler = on_track_handler.lock().await;
@@ -584,11 +578,6 @@ impl PeerConnection {
                 return Err(Error::ErrModifyingCertificates.into());
             }
 
-            /*TODO: for (i, certificate) in configuration.certificates.iter().enumerate() {
-                if !self.configuration.certificates[i].Equals(certificate) {
-                    return Err(Error::ErrModifyingCertificates.into());
-                }
-            }*/
             self.configuration.certificates = configuration.certificates;
         }
 
@@ -728,13 +717,13 @@ impl PeerConnection {
                 }
             }
 
-            let has_current_remote_description = {
+            let current_remote_description_is_none = {
                 let current_remote_description =
                     self.internal.current_remote_description.lock().await;
                 current_remote_description.is_none()
             };
 
-            let mut d = if !has_current_remote_description {
+            let mut d = if current_remote_description_is_none {
                 self.internal
                     .generate_unmatched_sdp(
                         current_transceivers,
@@ -1276,7 +1265,7 @@ impl PeerConnection {
                                         kind,
                                         Arc::clone(&self.internal.dtls_transport),
                                         Arc::clone(&self.internal.media_engine),
-                                        self.internal.interceptor.clone(),
+                                        Arc::clone(&self.internal.interceptor),
                                     ));
 
                                     let local_direction =
@@ -1286,14 +1275,15 @@ impl PeerConnection {
                                             RTPTransceiverDirection::Recvonly
                                         };
 
-                                    let t = Arc::new(RTPTransceiver::new(
+                                    let t = RTPTransceiver::new(
                                         Some(receiver),
                                         None,
                                         local_direction,
                                         kind,
                                         vec![],
                                         Arc::clone(&self.internal.media_engine),
-                                    ));
+                                    )
+                                    .await;
 
                                     self.internal.add_rtp_transceiver(Arc::clone(&t)).await;
 
@@ -1364,8 +1354,7 @@ impl PeerConnection {
                 }
             }
 
-            let (fingerprint, fingerprint_hash) = (String::new(), String::new());
-            //TODO: let (_fingerprint, _fingerprint_hash) = extract_fingerprint(parsed)?;
+            let (fingerprint, fingerprint_hash) = extract_fingerprint(parsed)?;
 
             // If one of the agents is lite and the other one is not, the lite agent must be the controlling agent.
             // If both or neither agents are lite the offering agent is controlling.
@@ -1385,6 +1374,8 @@ impl PeerConnection {
                 self.start_rtp_senders().await?;
             }
 
+            //log::trace!("start_transports: parsed={:?}", parsed);
+
             let pci = Arc::clone(&self.internal);
             let sdp_semantics = self.configuration.sdp_semantics;
             let dtls_role = DTLSRole::from(parsed);
@@ -1402,7 +1393,7 @@ impl PeerConnection {
                         log::trace!(
                             "start_transports: ice_role={}, dtls_role={}",
                             ice_role,
-                            dtls_role
+                            dtls_role,
                         );
                         pc.start_transports(ice_role, dtls_role, ru, rp, fp, fp_hash)
                             .await;
@@ -1520,12 +1511,15 @@ impl PeerConnection {
             let rtp_transceivers = self.internal.rtp_transceivers.lock().await;
             for t in &*rtp_transceivers {
                 if !t.stopped && t.kind == track.kind() && t.sender().await.is_none() {
-                    let sender = Arc::new(RTPSender::new(
-                        Arc::clone(&track),
-                        Arc::clone(&self.internal.dtls_transport),
-                        Arc::clone(&self.internal.media_engine),
-                        self.internal.interceptor.clone(),
-                    ));
+                    let sender = Arc::new(
+                        RTPSender::new(
+                            Arc::clone(&track),
+                            Arc::clone(&self.internal.dtls_transport),
+                            Arc::clone(&self.internal.media_engine),
+                            Arc::clone(&self.internal.interceptor),
+                        )
+                        .await,
+                    );
 
                     if let Err(err) = t
                         .set_sender_track(Some(Arc::clone(&sender)), Some(Arc::clone(&track)))
@@ -1569,7 +1563,8 @@ impl PeerConnection {
 
         let transceiver = self
             .internal
-            .new_transceiver_from_track(RTPTransceiverDirection::Sendrecv, track)?;
+            .new_transceiver_from_track(RTPTransceiverDirection::Sendrecv, track)
+            .await?;
         self.internal
             .add_rtp_transceiver(Arc::clone(&transceiver))
             .await;
@@ -1655,7 +1650,8 @@ impl PeerConnection {
 
         let t = self
             .internal
-            .new_transceiver_from_track(direction, Arc::clone(track))?;
+            .new_transceiver_from_track(direction, Arc::clone(track))
+            .await?;
 
         self.internal.add_rtp_transceiver(Arc::clone(&t)).await;
 
@@ -1770,12 +1766,12 @@ impl PeerConnection {
 
     /// write_rtcp sends a user provided RTCP packet to the connected peer. If no peer is connected the
     /// packet is discarded. It also runs any configured interceptors.
-    pub async fn write_rtcp(&self, pkts: &dyn rtcp::packet::Packet) -> Result<()> {
-        if let Some(rtc_writer) = &self.interceptor_rtcp_writer {
-            let a = Attributes::new();
-            rtc_writer.write(pkts, &a).await?;
-        }
-        Ok(())
+    pub async fn write_rtcp(
+        &self,
+        pkt: &(dyn rtcp::packet::Packet + Send + Sync),
+    ) -> Result<usize> {
+        let a = Attributes::new();
+        self.interceptor_rtcp_writer.write(pkt, &a).await
     }
 
     /// close ends the PeerConnection
@@ -1801,10 +1797,8 @@ impl PeerConnection {
         //    continue the chain the Mux has to be closed.
         let mut close_errs = vec![];
 
-        if let Some(interceptor) = &self.internal.interceptor {
-            if let Err(err) = interceptor.close().await {
-                close_errs.push(err);
-            }
+        if let Err(err) = self.internal.interceptor.close().await {
+            close_errs.push(err);
         }
 
         // https://www.w3.org/TR/webrtc/#dom-rtcpeerconnection-close (step #4)
@@ -2010,23 +2004,25 @@ impl PeerConnection {
         // It's possible to miss the GatherComplete event since setGatherCompleteHandler is an atomic operation and the
         // promise might have been created after the gathering is finished. Therefore, we need to check if the ICE gathering
         // state has changed to complete so that we don't block the caller forever.
-        let ice_gatherer = Arc::clone(&self.internal.ice_gatherer);
-        let mut done = Some(gathering_complete_tx);
+        let done = Arc::new(Mutex::new(Some(gathering_complete_tx)));
+        let done2 = Arc::clone(&done);
         self.internal
             .set_gather_complete_handler(Box::new(move || {
-                let state = match ice_gatherer.state() {
-                    ICEGathererState::New => ICEGatheringState::New,
-                    ICEGathererState::Gathering => ICEGatheringState::Gathering,
-                    _ => ICEGatheringState::Complete,
-                };
-                if state == ICEGatheringState::Complete {
-                    log::trace!("ICEGatheringState::Complete");
-                    done.take();
-                }
-                Box::pin(async move {})
+                log::trace!("setGatherCompleteHandler");
+                let done3 = Arc::clone(&done2);
+                Box::pin(async move {
+                    let mut d = done3.lock().await;
+                    d.take();
+                })
             }))
             .await;
 
+        if self.ice_gathering_state() == ICEGatheringState::Complete {
+            log::trace!("ICEGatheringState::Complete");
+            let mut d = done.lock().await;
+            d.take();
+        }
+
         gathering_complete_rx
     }
 }
