@@ -1,6 +1,5 @@
 use super::*;
 
-#[derive(Default, Clone)]
 pub(crate) struct PeerConnectionInternal {
     pub(super) on_negotiation_needed_handler: Arc<Mutex<Option<OnNegotiationNeededHdlrFn>>>,
     pub(super) is_closed: Arc<AtomicBool>,
@@ -39,18 +38,31 @@ pub(crate) struct PeerConnectionInternal {
     // A reference to the associated API state used by this connection
     pub(super) setting_engine: Arc<SettingEngine>,
     pub(crate) media_engine: Arc<MediaEngine>,
-    pub(super) interceptor: Option<Arc<dyn Interceptor + Send + Sync>>,
+    pub(super) interceptor: Arc<dyn Interceptor + Send + Sync>,
 }
 
 impl PeerConnectionInternal {
-    pub(super) async fn new(api: &API, configuration: &Configuration) -> Result<Self> {
+    pub(super) async fn new(api: &API, configuration: &mut Configuration) -> Result<Self> {
         let mut pc = PeerConnectionInternal {
+            on_negotiation_needed_handler: Arc::new(Default::default()),
             ops: Arc::new(Operations::new()),
             is_closed: Arc::new(AtomicBool::new(false)),
             is_negotiation_needed: Arc::new(AtomicBool::new(false)),
             negotiation_needed_state: Arc::new(AtomicU8::new(NegotiationNeededState::Empty as u8)),
             signaling_state: Arc::new(AtomicU8::new(SignalingState::Stable as u8)),
+            ice_transport: Arc::new(Default::default()),
+            dtls_transport: Arc::new(Default::default()),
             ice_connection_state: Arc::new(AtomicU8::new(ICEConnectionState::New as u8)),
+            sctp_transport: Arc::new(Default::default()),
+            rtp_transceivers: Arc::new(Default::default()),
+            on_track_handler: Arc::new(Default::default()),
+            on_signaling_state_change_handler: Arc::new(Default::default()),
+            on_ice_connection_state_change_handler: Arc::new(Default::default()),
+            on_data_channel_handler: Arc::new(Default::default()),
+            ice_gatherer: Arc::new(Default::default()),
+            current_local_description: Arc::new(Default::default()),
+            current_remote_description: Arc::new(Default::default()),
+            pending_local_description: Arc::new(Default::default()),
             peer_connection_state: Arc::new(AtomicU8::new(PeerConnectionState::New as u8)),
 
             setting_engine: Arc::clone(&api.setting_engine),
@@ -59,9 +71,9 @@ impl PeerConnectionInternal {
             } else {
                 Arc::clone(&api.media_engine)
             },
-            interceptor: api.interceptor.clone(),
-
-            ..Default::default()
+            interceptor: Arc::clone(&api.interceptor),
+            on_peer_connection_state_change_handler: Arc::new(Default::default()),
+            pending_remote_description: Arc::new(Default::default()),
         };
 
         // Create the ice gatherer
@@ -74,10 +86,9 @@ impl PeerConnectionInternal {
         pc.ice_transport = pc.create_ice_transport(api).await;
 
         // Create the DTLS transport
-        pc.dtls_transport = Arc::new(api.new_dtls_transport(
-            Arc::clone(&pc.ice_transport),
-            configuration.certificates.clone(),
-        )?);
+        let certificates = configuration.certificates.drain(..).collect();
+        pc.dtls_transport =
+            Arc::new(api.new_dtls_transport(Arc::clone(&pc.ice_transport), certificates)?);
 
         // Create the SCTP transport
         pc.sctp_transport = Arc::new(api.new_sctp_transport(Arc::clone(&pc.dtls_transport))?);
@@ -96,8 +107,6 @@ impl PeerConnectionInternal {
             }))
             .await;
 
-        //TODO: pc.interceptor_rtcp_writer = api.interceptor.bind_rtcp_writer(interceptor.RTCPWriterFunc(pc.writeRTCP))
-
         Ok(pc)
     }
 
@@ -139,7 +148,7 @@ impl PeerConnectionInternal {
                         receiver.kind(),
                         Arc::clone(&self.dtls_transport),
                         Arc::clone(&self.media_engine),
-                        self.interceptor.clone(),
+                        Arc::clone(&self.interceptor),
                     ));
                     t.set_receiver(Some(receiver)).await;
                 }
@@ -424,24 +433,25 @@ impl PeerConnectionInternal {
                     math_rand_alpha(16),
                 ));
 
-                self.new_transceiver_from_track(direction, track)?
+                self.new_transceiver_from_track(direction, track).await?
             }
             RTPTransceiverDirection::Recvonly => {
                 let receiver = Arc::new(RTPReceiver::new(
                     kind,
                     Arc::clone(&self.dtls_transport),
                     Arc::clone(&self.media_engine),
-                    self.interceptor.clone(),
+                    Arc::clone(&self.interceptor),
                 ));
 
-                Arc::new(RTPTransceiver::new(
+                RTPTransceiver::new(
                     Some(receiver),
                     None,
                     RTPTransceiverDirection::Recvonly,
                     kind,
                     vec![],
                     Arc::clone(&self.media_engine),
-                ))
+                )
+                .await
             }
             _ => return Err(Error::ErrPeerConnAddTransceiverFromKindSupport.into()),
         };
@@ -451,7 +461,7 @@ impl PeerConnectionInternal {
         Ok(t)
     }
 
-    pub(super) fn new_transceiver_from_track(
+    pub(super) async fn new_transceiver_from_track(
         &self,
         direction: RTPTransceiverDirection,
         track: Arc<dyn TrackLocal + Send + Sync>,
@@ -462,36 +472,43 @@ impl PeerConnectionInternal {
                     track.kind(),
                     Arc::clone(&self.dtls_transport),
                     Arc::clone(&self.media_engine),
-                    self.interceptor.clone(),
-                )));
-                let s = Some(Arc::new(RTPSender::new(
-                    Arc::clone(&track),
-                    Arc::clone(&self.dtls_transport),
-                    Arc::clone(&self.media_engine),
-                    self.interceptor.clone(),
+                    Arc::clone(&self.interceptor),
                 )));
+                let s = Some(Arc::new(
+                    RTPSender::new(
+                        Arc::clone(&track),
+                        Arc::clone(&self.dtls_transport),
+                        Arc::clone(&self.media_engine),
+                        Arc::clone(&self.interceptor),
+                    )
+                    .await,
+                ));
                 (r, s)
             }
             RTPTransceiverDirection::Sendonly => {
-                let s = Some(Arc::new(RTPSender::new(
-                    Arc::clone(&track),
-                    Arc::clone(&self.dtls_transport),
-                    Arc::clone(&self.media_engine),
-                    self.interceptor.clone(),
-                )));
+                let s = Some(Arc::new(
+                    RTPSender::new(
+                        Arc::clone(&track),
+                        Arc::clone(&self.dtls_transport),
+                        Arc::clone(&self.media_engine),
+                        Arc::clone(&self.interceptor),
+                    )
+                    .await,
+                ));
                 (None, s)
             }
             _ => return Err(Error::ErrPeerConnAddTransceiverFromTrackSupport.into()),
         };
 
-        Ok(Arc::new(RTPTransceiver::new(
+        Ok(RTPTransceiver::new(
             r,
             s,
             direction,
             track.kind(),
             vec![],
             Arc::clone(&self.media_engine),
-        )))
+        )
+        .await)
     }
 
     /// add_rtp_transceiver appends t into rtp_transceivers
@@ -533,14 +550,6 @@ impl PeerConnectionInternal {
         self.ice_gatherer.on_gathering_complete(f).await;
     }
 
-    async fn dtls_write_rtcp(
-        &self,
-        pkts: &dyn rtcp::packet::Packet,
-        _a: &Attributes,
-    ) -> Result<usize> {
-        self.dtls_transport.write_rtcp(pkts).await
-    }
-
     /// Start all transports. PeerConnection now has enough state
     pub(super) async fn start_transports(
         self: &Arc<Self>,
@@ -555,7 +564,7 @@ impl PeerConnectionInternal {
         if let Err(err) = self
             .ice_transport
             .start(
-                ICEParameters {
+                &ICEParameters {
                     username_fragment: remote_ufrag,
                     password: remote_pwd,
                     ice_lite: false,
@@ -564,7 +573,7 @@ impl PeerConnectionInternal {
             )
             .await
         {
-            log::warn!("Failed to start manager: {}", err);
+            log::warn!("Failed to start manager ice: {}", err);
             return;
         }
 
@@ -588,7 +597,7 @@ impl PeerConnectionInternal {
         )
         .await;
         if let Err(err) = result {
-            log::warn!("Failed to start manager: {}", err);
+            log::warn!("Failed to start manager dtls: {}", err);
         }
     }
 
@@ -682,11 +691,11 @@ impl PeerConnectionInternal {
             }
         }
 
-        let dtls_fingerprints = vec![];
-        /*TODO: dtls_fingerprints, err := self.configuration.Certificates[0].GetFingerprints()
-        if err != nil {
-            return nil, err
-        }*/
+        let dtls_fingerprints = if let Some(cert) = self.dtls_transport.certificates.first() {
+            vec![cert.get_fingerprint()?]
+        } else {
+            return Err(Error::ErrNonCertificate.into());
+        };
 
         let params = PopulateSdpParams {
             is_plan_b,
@@ -795,8 +804,9 @@ impl PeerConnectionInternal {
                                             kind,
                                             vec![],
                                             Arc::clone(&self.media_engine),
-                                        );
-                                        media_transceivers.push(Arc::new(t));
+                                        )
+                                        .await;
+                                        media_transceivers.push(t);
                                     }
                                     break;
                                 }
@@ -876,11 +886,11 @@ impl PeerConnectionInternal {
             log::info!("Plan-B Offer detected; responding with Plan-B Answer");
         }
 
-        let dtls_fingerprints = vec![];
-        /*TODO:dtls_fingerprints, err := self.configuration.Certificates[0].GetFingerprints()
-        if err != nil {
-            return nil, err
-        }*/
+        let dtls_fingerprints = if let Some(cert) = self.dtls_transport.certificates.first() {
+            vec![cert.get_fingerprint()?]
+        } else {
+            return Err(Error::ErrNonCertificate.into());
+        };
 
         let params = PopulateSdpParams {
             is_plan_b: detected_plan_b,
@@ -1158,3 +1168,14 @@ impl PeerConnectionInternal {
         false
     }
 }
+
+#[async_trait]
+impl RTCPWriter for PeerConnectionInternal {
+    async fn write(
+        &self,
+        pkt: &(dyn rtcp::packet::Packet + Send + Sync),
+        _a: &Attributes,
+    ) -> Result<usize> {
+        self.dtls_transport.write_rtcp(pkt).await
+    }
+}
